# -*- coding: utf-8 -*-
from .my_application_ch import MyApplication as myApp_ch
from .my_application_en import MyApplication as myApp_en
from .my_application_json import MyApplication as myApp_json
from .base.my_application import MyApplication as myApp_base


class MyApplicationDispatch(object):
    @staticmethod
    def list(auth, format_json):
        if format_json:
            myApp_json.list(auth)
        else:
            if auth.language == "en":
                myApp_en.list(auth)
            else:
                myApp_ch.list(auth)

    @staticmethod
    def export(auth, appid, apkname, version, description):
        myApp_base._export(auth, appid, apkname, version, description)

    @staticmethod
    def _import(auth, apkid, clusterid, applicationname):
        myApp_base._import(auth, apkid, clusterid, applicationname)



# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.file import FileDispatch as File


class MoveCommands(object):
    @args("--path", dest="paths", nargs="+", required=True, help="full path of file")
    @args("--newdir", dest="newdir", required=True, help="valid dir")
    def file(self, auth, paths, newdir):
        """copy files"""
        File.move_batch(auth, paths, newdir)




# -*- coding: utf-8 -*-
import collections
from ..utils.common_utils import print_table, obj_replace_time_field
from .base.my_application import MyApplication as myApp_base

buildStatus_ch = {"SUCCESS": u"可用",
                  "FAILED": u"不可用",
                  "ING": u"打包中"}


class MyApplication(object):
    @staticmethod
    def list(auth):
        try:
            data = myApp_base._list(auth)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = u"应用名称"
            header["versionName"] = u"版本名称"
            header["buildStatus"] = u"状态"
            header["createdTime"] = u"创建时间"
            application_pks_info = []
            for _ in data:
                for application_pk in _.get("applicationPks", []):
                    application_pk = obj_replace_time_field(application_pk, ["createdTime"])
                    application_pk["buildStatus"] = buildStatus_ch.get(application_pk.get("buildStatus", ""), u"")
                    application_pk_info = [application_pk.get(key, "") for key in header]
                    application_pks_info.append(application_pk_info)
            print_table(header.values(), application_pks_info, auth.encoding)
        except Exception as e:
            raise




# -*- coding: utf-8 -*-
import json
import os
from ...utils.http_utils import http_request
from requests_toolbelt.multipart.encoder import MultipartEncoder


class File(object):
    @staticmethod
    def _list(auth, path):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/fileservice/v1/file")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token, "Content-Type": "application/json"}
        request_body = json.dumps(dict(action="list",
                                       path=path if path.startswith("/") else "/{}".format(path)))
        status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        return data

    @staticmethod
    def _upload(auth, dir, files):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/fileservice/v1/file")
        http_method = "POST"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token}
        file_name = "file"
        body_dict = {}
        for index, _file in enumerate(files):
            if os.access(_file, os.R_OK):
                body_dict["{}-{}".format(file_name, index)] = (os.path.basename(_file), open(_file, 'rb'), 'application/octet-stream')
            else:
                raise Exception("File Error")
        body_dict["destination"] = dir
        multipart_encoder = MultipartEncoder(
            fields=body_dict
        )
        headers['Content-Type'] = multipart_encoder.content_type
        status, response_data = http_request(url=url, http_method=http_method, body=multipart_encoder, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        result = data.get("result",{})
        if result.get("success"):
            print "Uploade files:{} success".format(",".join(files))
        else:
            print result.get("error")

    @staticmethod
    def _download(auth, path):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        filename = os.path.basename(path)
        url = "{}://{}{}?action=download&path={}&Authorization={}&X-Access-Authority={}".format(
            security, address, "/dispatch/fileservice/v1/file", path, auth_token, access_authority_token)
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        status, content = http_request(url=url, http_method=http_method, headers=headers)
        if status == 200:
            with open(filename, 'wb') as f:
                f.write(content)
            print "success"
        else:
            data = json.loads(content)
            message = data.get("message", "Unknown Error")
            raise Exception(message)

    @staticmethod
    def _delete_batch(auth, paths):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/fileservice/v1/file")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(action="remove",
                                       items=[path if path.startswith("/") else "/{}".format(path) for path in paths]))
        status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        result = data.get("result", {})
        if result.get("success"):
            print "Delete files:{} success".format(",".join(paths))
        else:
            print result.get("error")

    @staticmethod
    def _move_batch(auth, paths, newdir):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/fileservice/v1/file")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(action="move",
                                       items=[path if path.startswith("/") else "/{}".format(path) for path in paths],
                                       newPath=newdir))
        status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        result = data.get("result", {})
        if result.get("success"):
            print "Move files:{} success".format(",".join(paths))
        else:
            print result.get("error")

    @staticmethod
    def _copy_batch(auth, paths, newdir):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/fileservice/v1/file")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(action="copy",
                                       items=[path if path.startswith("/") else "/{}".format(path) for path in paths],
                                       newPath=newdir))
        status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        result = data.get("result", {})
        if result.get("success"):
            print "Copy files:{} success".format(",".join(paths))
        else:
            print result.get("error")

    @staticmethod
    def _rename(auth, path, newpath):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/fileservice/v1/file")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(action="rename",
                                       item=path if path.startswith("/") else "/{}".format(path),
                                       newItemPath=newpath if newpath.startswith("/") else "/{}".format(newpath)))
        status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        result = data.get("result", {})
        if result.get("success"):
            print "Rename files:{} success".format(path)
        else:
            print result.get("error")



# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.my_application import MyApplicationDispatch as myApp


class ExportCommands(object):
    @args("--appid", dest="appid", required=True, help="appid of exported app")
    @args("--name", dest="apkname", required=True, help="apk name")
    @args("--version", dest="version", required=True, help="apk version")
    @args("--description", dest="description", default="", help="apk description")
    def myapp(self, auth, appid, apkname, version, description):
        """export app to apk"""
        myApp.export(auth, appid, apkname, version, description)




# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.service import ServiceDispatch as Service


class RedeployCommands(object):
    @args("--serviceid", dest="serviceids", required=True, nargs="+", help="")
    @args("--appid", dest="appid", required=True, help="")
    @args("--timeout", dest="timeout", type=int, default=200, help='set timeout when sync is True')
    @args("--sync", dest="sync", type=bool, default=False, choices=[True, False], help='wait for callback when sync is True')
    def service(self, auth, serviceids, appid, timeout=200, sync=False):
        """redeploy service"""
        if sync and not timeout:
            raise Exception("please set timeout when sync")
        Service.redeploy_batch(auth, appid, serviceids, timeout, sync)





# -*- coding: utf-8 -*-
import collections
from ..utils.common_utils import print_table, obj_replace_time_field
from .base.my_application import MyApplication as myApp_base


buildStatus_en = {"SUCCESS": "Success",
                  "FAILED": "Failed",
                  "ING": "Building"}


class MyApplication(object):
    @staticmethod
    def list(auth):
        try:
            data = myApp_base._list(auth)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = "Name"
            header["versionName"] = "Version"
            header["buildStatus"] = "Build Status"
            header["createdTime"] = "Create Time"
            application_pks_info = []
            for _ in data:
                for application_pk in _.get("applicationPks", []):
                    application_pk = obj_replace_time_field(application_pk, ["createdTime"])
                    application_pk["buildStatus"] = buildStatus_en.get(application_pk.get("buildStatus", ""), "")
                    application_pk_info = [application_pk.get(key, "") for key in header]
                    application_pks_info.append(application_pk_info)
            print_table(header.values(), application_pks_info, auth.encoding)
        except Exception as e:
            raise




# -*- coding: utf-8 -*-
from .service_ch import Service as Srv_ch
from .service_en import Service as Srv_en
from .service_json import Service as Srv_json
from .base.service import Service as Srv_base
from .base.common import Common


class ServiceDispatch(object):
    @staticmethod
    def list(auth, appid, format_json):
        """根据应用ID显示该应用下服务列表"""
        if format_json:
            Srv_json.list(auth, appid)
        else:
            if auth.language == "en":
                Srv_en.list(auth, appid)
            else:
                Srv_ch.list(auth, appid)

    @staticmethod
    def deploy_batch(auth, appid, serviceids, timeout, sync):
        """服务上线"""
        job_id = Srv_base._deploy_batch(auth, appid, serviceids)
        if sync:
            status = Common._job_check(timeout, auth, job_id)
            if status == "success":
                print "deploy {} successfully".format(','.join(serviceids))
            else:
                raise Exception("deploy {} fail".format(','.join(serviceids)))

    @staticmethod
    def redeploy_batch(auth, appid, serviceids, timeout, sync):
        """服务上线"""
        job_id = Srv_base._redeploy_batch(auth, appid, serviceids)
        if sync:
            status = Common._job_check(timeout, auth, job_id)
            if status == "success":
                print "redeploy {} successfully".format(','.join(serviceids))
            else:
                raise Exception("redeploy {} fail".format(','.join(serviceids)))

    @staticmethod
    def undeploy_batch(auth, appid, serviceids, timeout, sync):
        """服务下线"""
        job_id = Srv_base._undeploy_batch(auth, appid, serviceids)
        if sync:
            status = Common._job_check(timeout, auth, job_id)
            if status == "success":
                print "undeploy {} successfully".format(','.join(serviceids))
            else:
                raise Exception("undeploy {} fail".format(','.join(serviceids)))

    @staticmethod
    def delete_batch(auth, appid, serviceids):
        """批量删除服务"""
        Srv_base._delete_batch(auth, appid, serviceids)

    @staticmethod
    def clean_data(auth, appid, serviceid):
        Srv_base._clean_data(auth, appid, serviceid)

    # @staticmethod
    # def delete(auth, serviceid):
    #     """删除服务"""
    #     print("delete service")



# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.my_application import MyApplicationDispatch as myApp


class ImportCommands(object):
    @args("--apkid", dest="apkid", required=True, help="id of imported apk")
    @args("--clusterid", dest="clusterid", required=True, help="select cluster to deploy")
    @args("--appname", dest="applicationname", required=True, help="application name")
    def myapp(self, auth, apkid, clusterid, applicationname):
        """import apk to app"""
        myApp._import(auth, apkid, clusterid, applicationname)




# -*- coding: utf-8 -*-
import json
from ...utils.http_utils import http_request


class MyApplication(object):
    @staticmethod
    def _list(auth):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/application/v1/application")
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token, "Content-Type": "application/json"}
        status, response_data = http_request(url=url, http_method=http_method, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        return data

    @staticmethod
    def _export(auth, appid, apkname, version, description):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/application/v1/application")
        http_method = "POST"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_data = json.dumps(dict(applicationId=appid,
                                        detail="",
                                        name=apkname,
                                        picture="fa fa-cloud te-bg-1",
                                        versionDescription=description,
                                        versionName=version))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_data, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u"Export application {} complete".format(appid))
        except Exception as e:
            raise

    @staticmethod
    def _import(auth, apkid, clusterid, applicationname):
        data = MyApplication._show(auth, apkid)
        application_file_path = data.get("zipUrl", "")
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/application/v1/application/import")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_data = json.dumps(dict(applicationName=applicationname,
                                       application_file_path=application_file_path,
                                       clusterId=clusterid,
                                       description="",
                                       tags=[]))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_data, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u"Import apk {} complete".format(apkid))
        except Exception as e:
            raise

    @staticmethod
    def _show(auth, apkid):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}/{}".format(security, address, "/dispatch/application/v1/application", apkid)
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        status, response_data = http_request(url=url, http_method=http_method, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Authorization Error")
            raise Exception(message)
        return data



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json
from .base.my_application import MyApplication as myApp_base


class MyApplication(object):
    @staticmethod
    def list(auth):
        try:
            data = myApp_base._list(auth)
            application_pks_info = []
            for _ in data:
                for application_pk in _.get("applicationPks", []):
                    application_pk_info = dict(id=application_pk.get("id", ""),
                                               name=application_pk.get("name", ""),
                                               version=application_pk.get("versionName", ""),
                                               buildStatus=application_pk.get("buildStatus", ""),
                                               createdTime=application_pk.get("createdTime", ""))
                    application_pks_info.append(application_pk_info)
            print json.dumps(dict(application_pks=application_pks_info), indent=4, ensure_ascii=False)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.file import FileDispatch as File


class RenameCommands(object):
    @args("--path", dest="path", required=True, help="full path of file")
    @args("--newpath", dest="newpath", required=True, help="full path of renamed file")
    def file(self, auth, path, newpath):
        """copy files"""
        File.rename(auth, path, newpath)




# -*- coding: utf-8 -*-
import json
import time
from ...utils.http_utils import http_request


class Common(object):
    @staticmethod
    def _job_check(timeout, auth, job_id):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}/{}".format(security, address, "/dispatch/platform/v1/jobs", job_id)
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        deadline = int(time.time()) + timeout
        while int(time.time()) < deadline:
            status, response_data = http_request(url=url, http_method=http_method, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print "total step:{}, current step:{}.".format(data.get("total_step", ""), data.get("step", ""))
            if data.get("status", None):
                return data.get("status")
            time.sleep(5)
        raise Exception("timeout")
